﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using IdeaBlade.EntityModel;

namespace WpfSimpleSteps
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private MainWindowViewModel viewModel;

        public MainWindow()
        {
            InitializeComponent();
        }

        private void EnableDisableButtons()
        {
            firstButton.IsEnabled = viewModel.CurrentPosition > 0;
            prevButton.IsEnabled = viewModel.CurrentPosition > 0;
            nextButton.IsEnabled = viewModel.CurrentPosition < viewModel.Employees.Count - 1;
            lastButton.IsEnabled = viewModel.CurrentPosition < viewModel.Employees.Count - 1;
            addButton.IsEnabled = viewModel.Employees != null;
            deleteButton.IsEnabled = viewModel.CurrentPosition > -1;

            okButton.IsEnabled = viewModel.HasChanges;
            cancelButton.IsEnabled = viewModel.HasChanges;
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            viewModel = new MainWindowViewModel();
            DataContext = viewModel;

            viewModel.PropertyChanged += delegate { EnableDisableButtons(); };
            viewModel.Employees.CollectionChanged += delegate { EnableDisableButtons(); };
        }

        private void firstButton_Click(object sender, RoutedEventArgs e)
        {
            viewModel.MoveCurrentToFirst();
        }

        private void prevButton_Click(object sender, RoutedEventArgs e)
        {
            viewModel.MoveCurrentToPrevious();
        }

        private void nextButton_Click(object sender, RoutedEventArgs e)
        {
            viewModel.MoveCurrentToNext();
        }

        private void lastButton_Click(object sender, RoutedEventArgs e)
        {
            viewModel.MoveCurrentToLast();
        }

        private void addButton_Click(object sender, RoutedEventArgs e)
        {
            viewModel.AddNew();
        }

        private void deleteButton_Click(object sender, RoutedEventArgs e)
        {
            if (viewModel.CurrentPosition > -1)
            {
                viewModel.RemoveAt(viewModel.CurrentPosition);
            }
        }

        private void okButton_Click(object sender, RoutedEventArgs e)
        {
            viewModel.Save();
        }

        private void cancelButton_Click(object sender, RoutedEventArgs e)
        {
            viewModel.RollbackChanges();
        }

        private void OrdersGrid_AutoGeneratingColumn(object sender, DataGridAutoGeneratingColumnEventArgs e)
        {
            e.Cancel = e.PropertyName == "Customer" || e.PropertyName == "OrderDetails";
        }
    }
}
