﻿Imports System.Collections.Specialized
Imports System.Text
Imports System.Collections.ObjectModel

Imports IdeaBlade.Core
Imports IdeaBlade.EntityModel
Imports IdeaBlade.Validation
Imports System.ComponentModel

Namespace WpfSimpleSteps
	Friend Class MainWindowViewModel
		Implements INotifyPropertyChanged
		Private privateEmployees As ObservableCollection(Of Employee)
		Public Property Employees() As ObservableCollection(Of Employee)
			Get
				Return privateEmployees
			End Get
			Private Set(ByVal value As ObservableCollection(Of Employee))
				privateEmployees = value
			End Set
		End Property
		Private privateLog As ObservableCollection(Of String)
		Public Property Log() As ObservableCollection(Of String)
			Get
				Return privateLog
			End Get
			Private Set(ByVal value As ObservableCollection(Of String))
				privateLog = value
			End Set
		End Property

    Public Sub New()
      AsyncMainWindowViewModel()
    End Sub

    Public Async Sub AsyncMainWindowViewModel()
      Log = New ObservableCollection(Of String)()
      Employees = New ObservableCollection(Of Employee)()
      AddHandler Employees.CollectionChanged, AddressOf Employees_CollectionChanged

      _mgr = New NorthwindIBEntities()
      AddHandler _mgr.EntityChanged, Sub(s, args) HasChanges = (CType(s, NorthwindIBEntities)).HasChanges()
      AddHandler _mgr.Fetching, AddressOf _mgr_Fetching

      '_query = _mgr.Employees.Include("Orders");
      _query = _mgr.Employees
      Dim results = Await _query.ExecuteAsync()
      results.ForEach(Sub(emp) Employees.Add(emp))

      WriteToLog("Initializing View Model")

      '_mgr.VerifierEngine.DefaultVerifierOptions.ErrorNotificationMode = VerifierErrorNotificationMode.NotifyAndThrowException;
    End Sub

		Private Sub _mgr_Fetching(ByVal sender As Object, ByVal e As EntityFetchingEventArgs)
      WriteToLog("Fetching " & e.Query.ElementType.ToString())
		End Sub

		Public Property CurrentEmployee() As Employee
			Get
				Return _currentEmployee
			End Get
			Private Set(ByVal value As Employee)
				_currentEmployee = value
				RaisePropertyChanged("CurrentEmployee")
			End Set
		End Property

		Public Property CurrentPosition() As Integer
			Get
				Return _currentPosition
			End Get
			Set(ByVal value As Integer)
				If Employees IsNot Nothing AndAlso Employees.Count > 0 Then
					_currentPosition = Math.Min(Math.Max(value, 0), Employees.Count - 1)
					CurrentEmployee = Employees(_currentPosition)
				Else
					_currentPosition = -1
					CurrentEmployee = Nothing
				End If
				RaisePropertyChanged("CurrentPosition")
			End Set
		End Property

		Public Property HasChanges() As Boolean
			Get
				Return _hasChanges
			End Get
			Private Set(ByVal value As Boolean)
				_hasChanges = value
				RaisePropertyChanged("HasChanges")
			End Set
		End Property

		Public Sub MoveCurrentToFirst()
			CurrentPosition = 0
		End Sub

		Public Sub MoveCurrentToPrevious()
			CurrentPosition = CurrentPosition - 1
		End Sub

		Public Sub MoveCurrentToNext()
			CurrentPosition = CurrentPosition + 1
		End Sub

		Public Sub MoveCurrentToLast()
			CurrentPosition = If(Employees IsNot Nothing, Employees.Count - 1, -1)
		End Sub

		Public Sub AddNew()
			Employees.Add(New Employee())
			MoveCurrentToLast()
		End Sub

		Public Sub RemoveAt(ByVal position As Integer)
			Employees.RemoveAt(position)
			CurrentPosition = position
		End Sub

		Public Sub Save()
			_mgr.SaveChanges()
		End Sub

		Public Sub RollbackChanges()
			Dim currentPosition As Integer = Me.CurrentPosition

			_mgr.RejectChanges()
			Employees.Clear()
      _query.With(QueryStrategy.CacheOnly).Execute().ForEach(Sub(emp) Employees.Add(emp))

			Me.CurrentPosition = currentPosition
		End Sub

		Private Sub RaisePropertyChanged(ByVal [property] As String)
			RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs([property]))
		End Sub

		Private Sub Employees_CollectionChanged(ByVal sender As Object, ByVal e As NotifyCollectionChangedEventArgs)
			Select Case e.Action
				Case NotifyCollectionChangedAction.Add
					Add(e.NewItems.Cast(Of Employee)())
				Case NotifyCollectionChangedAction.Remove
					Delete(e.OldItems.Cast(Of Employee)())
			End Select
		End Sub

		Private Sub Add(ByVal employees As IEnumerable(Of Employee))
      employees.ForEach(Sub(emp) _mgr.AddEntity(emp))
		End Sub

		Private Sub Delete(ByVal employees As IEnumerable(Of Employee))
      employees.ForEach(Sub(emp) emp.EntityAspect.Delete())
			Save()
		End Sub

		Private Sub WriteToLog(ByVal message As String)
			Log.Insert(0, message)
		End Sub

		Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

		Private _currentEmployee As Employee = Nothing
		Private _currentPosition As Integer = -1
    Private _mgr As NorthwindIBEntities
		Private _hasChanges As Boolean = False
    Private _query As EntityQuery(Of Employee)
	End Class
End Namespace
